import * as tslib_1 from "tslib";
import { Component, Model, Prop, Watch, Vue } from 'vue-property-decorator';
import VueSliderDot from './vue-slider-dot';
import VueSliderMark from './vue-slider-mark';
import { getSize, getPos, getKeyboardHandleFunc } from './utils';
import Decimal from './utils/decimal';
import Control from './utils/control';
import State from './utils/state';
import './styles/slider.scss';
export const SliderState = {
    None: 0,
    Drag: 1 << 0,
    Focus: 2 << 0,
};
const DEFAULT_SLIDER_SIZE = 4;
let VueSlider = class VueSlider extends Vue {
    constructor() {
        super(...arguments);
        this.states = new State(SliderState);
        // The width of the component is divided into one hundred, the width of each one.
        this.scale = 1;
        // Currently dragged slider index
        this.focusDotIndex = 0;
    }
    get tailSize() {
        return getSize((this.isHorizontal ? this.height : this.width) || DEFAULT_SLIDER_SIZE);
    }
    get containerClasses() {
        return [
            'vue-slider',
            [`vue-slider-${this.direction}`],
            {
                'vue-slider-disabled': this.disabled,
            },
        ];
    }
    get containerStyles() {
        const [dotWidth, dotHeight] = Array.isArray(this.dotSize)
            ? this.dotSize
            : [this.dotSize, this.dotSize];
        const containerWidth = this.width
            ? getSize(this.width)
            : this.isHorizontal
                ? 'auto'
                : getSize(DEFAULT_SLIDER_SIZE);
        const containerHeight = this.height
            ? getSize(this.height)
            : this.isHorizontal
                ? getSize(DEFAULT_SLIDER_SIZE)
                : 'auto';
        return {
            padding: this.isHorizontal ? `${dotHeight / 2}px 0` : `0 ${dotWidth / 2}px`,
            width: containerWidth,
            height: containerHeight,
        };
    }
    get processBaseStyleArray() {
        return this.control.processArray.map(([start, end, style]) => {
            if (start > end) {
                /* tslint:disable:semicolon */
                ;
                [start, end] = [end, start];
            }
            const sizeStyleKey = this.isHorizontal ? 'width' : 'height';
            return {
                [this.isHorizontal ? 'height' : 'width']: '100%',
                [this.isHorizontal ? 'top' : 'left']: 0,
                [this.mainDirection]: `${start}%`,
                [sizeStyleKey]: `${end - start}%`,
                transitionProperty: `${sizeStyleKey},${this.mainDirection}`,
                transitionDuration: `${this.animateTime}s`,
                ...style,
            };
        });
    }
    get dotBaseStyle() {
        const [dotWidth, dotHeight] = Array.isArray(this.dotSize)
            ? this.dotSize
            : [this.dotSize, this.dotSize];
        let dotPos;
        if (this.isHorizontal) {
            dotPos = {
                transform: `translate(${this.isReverse ? '50%' : '-50%'}, -50%)`,
                WebkitTransform: `translate(${this.isReverse ? '50%' : '-50%'}, -50%)`,
                top: '50%',
                [this.direction === 'ltr' ? 'left' : 'right']: '0',
            };
        }
        else {
            dotPos = {
                transform: `translate(-50%, ${this.isReverse ? '50%' : '-50%'})`,
                WebkitTransform: `translate(-50%, ${this.isReverse ? '50%' : '-50%'})`,
                left: '50%',
                [this.direction === 'btt' ? 'bottom' : 'top']: '0',
            };
        }
        return {
            width: `${dotWidth}px`,
            height: `${dotHeight}px`,
            ...dotPos,
        };
    }
    get mainDirection() {
        switch (this.direction) {
            case 'ltr':
                return 'left';
            case 'rtl':
                return 'right';
            case 'btt':
                return 'bottom';
            case 'ttb':
                return 'top';
        }
    }
    get isHorizontal() {
        return this.direction === 'ltr' || this.direction === 'rtl';
    }
    get isReverse() {
        return this.direction === 'rtl' || this.direction === 'btt';
    }
    get tooltipDirections() {
        const dir = this.tooltipPlacement || (this.isHorizontal ? 'top' : 'left');
        if (Array.isArray(dir)) {
            return dir;
        }
        else {
            return Array.from(new Array(this.dots.length), () => dir);
        }
    }
    get dots() {
        return this.control.dotsPos.map((pos, index) => ({
            pos,
            index,
            value: this.control.dotsValue[index],
            focus: this.states.has(SliderState.Focus) && this.focusDotIndex === index,
            disabled: false,
            style: this.dotStyle,
            ...((Array.isArray(this.dotOptions) ? this.dotOptions[index] : this.dotOptions) || {}),
        }));
    }
    get animateTime() {
        if (this.states.has(SliderState.Drag)) {
            return 0;
        }
        return this.duration;
    }
    get canSort() {
        return this.order && !this.minRange && !this.maxRange && !this.fixed && this.enableCross;
    }
    onValueChanged() {
        if (!this.states.has(SliderState.Drag) && this.isNotSync) {
            this.control.setValue(this.value);
        }
    }
    created() {
        this.initControl();
    }
    mounted() {
        this.bindEvent();
    }
    beforeDestroy() {
        this.unbindEvent();
    }
    bindEvent() {
        document.addEventListener('touchmove', this.dragMove, { passive: false });
        document.addEventListener('touchend', this.dragEnd, { passive: false });
        document.addEventListener('mousedown', this.blurHandle);
        document.addEventListener('mousemove', this.dragMove);
        document.addEventListener('mouseup', this.dragEnd);
        document.addEventListener('mouseleave', this.dragEnd);
        document.addEventListener('keydown', this.keydownHandle);
    }
    unbindEvent() {
        document.removeEventListener('touchmove', this.dragMove);
        document.removeEventListener('touchend', this.dragEnd);
        document.removeEventListener('mousemove', this.dragMove);
        document.removeEventListener('mouseup', this.dragEnd);
        document.removeEventListener('mouseleave', this.dragEnd);
        document.removeEventListener('keydown', this.keydownHandle);
    }
    setScale() {
        this.scale = new Decimal(Math.floor(this.isHorizontal ? this.$el.offsetWidth : this.$el.offsetHeight))
            .divide(100)
            .toNumber();
    }
    initControl() {
        this.control = new Control({
            value: this.value,
            data: this.data,
            enableCross: this.enableCross,
            fixed: this.fixed,
            max: this.max,
            min: this.min,
            interval: this.interval,
            minRange: this.minRange,
            maxRange: this.maxRange,
            order: this.order,
            marks: this.marks,
            process: this.process,
            onError: this.emitError,
        });
        [
            'data',
            'enableCross',
            'fixed',
            'max',
            'min',
            'interval',
            'minRange',
            'maxRange',
            'order',
            'marks',
            'process',
        ].forEach(name => {
            this.$watch(name, (val) => {
                if (name === 'data' &&
                    Array.isArray(this.control.data) &&
                    Array.isArray(val) &&
                    this.control.data.length === val.length &&
                    val.every((item, index) => item === this.control.data[index])) {
                    return false;
                }
                ;
                this.control[name] = val;
                if (['data', 'max', 'min', 'interval'].includes(name)) {
                    this.control.syncDotsPos();
                }
            });
        });
    }
    isDisabledByDotIndex(index) {
        return this.dots[index].disabled;
    }
    syncValueByPos() {
        let values = this.control.dotsValue;
        // When included is true, the return value is the value of the nearest mark
        if (this.included && this.control.markList.length > 0) {
            const getRecentValue = (val) => {
                let curValue = val;
                let dir = this.max - this.min;
                this.control.markList.forEach(mark => {
                    if (typeof mark.value === 'number' && typeof val === 'number') {
                        const curDir = Math.abs(mark.value - val);
                        if (curDir < dir) {
                            dir = curDir;
                            curValue = mark.value;
                        }
                    }
                });
                return curValue;
            };
            values = values.map(val => getRecentValue(val));
        }
        if (this.isDiff(values, Array.isArray(this.value) ? this.value : [this.value])) {
            this.$emit('change', values.length === 1 ? values[0] : [...values]);
        }
    }
    // Slider value and component internal value are inconsistent
    get isNotSync() {
        const values = this.control.dotsValue;
        return Array.isArray(this.value)
            ? this.value.length !== values.length ||
                this.value.some((val, index) => val !== values[index])
            : this.value !== values[0];
    }
    isDiff(value1, value2) {
        return value1.length !== value2.length || value1.some((val, index) => val !== value2[index]);
    }
    emitError(type, message) {
        this.$emit('error', type, message);
    }
    /**
     * Get the drag range of the slider
     *
     * @private
     * @param {number} index slider index
     * @returns {[number, number]} range [start, end]
     * @memberof VueSlider
     */
    get dragRange() {
        const prevDot = this.dots[this.focusDotIndex - 1];
        const nextDot = this.dots[this.focusDotIndex + 1];
        return [prevDot ? prevDot.pos : -Infinity, nextDot ? nextDot.pos : Infinity];
    }
    dragStart(index) {
        this.focusDotIndex = index;
        this.setScale();
        this.states.add(SliderState.Drag);
        this.states.add(SliderState.Focus);
        this.$emit('drag-start');
    }
    dragMove(e) {
        if (!this.states.has(SliderState.Drag)) {
            return false;
        }
        e.preventDefault();
        const pos = this.getPosByEvent(e);
        this.isCrossDot(pos);
        this.control.setDotPos(pos, this.focusDotIndex);
        if (!this.lazy) {
            this.syncValueByPos();
        }
        const value = this.control.dotsValue;
        this.$emit('dragging', value.length === 1 ? value[0] : [...value]);
    }
    // If the component is sorted, then when the slider crosses, toggle the currently selected slider index
    isCrossDot(pos) {
        if (this.canSort) {
            const curIndex = this.focusDotIndex;
            let curPos = pos;
            if (curPos > this.dragRange[1]) {
                curPos = this.dragRange[1];
                this.focusDotIndex++;
            }
            else if (curPos < this.dragRange[0]) {
                curPos = this.dragRange[0];
                this.focusDotIndex--;
            }
            if (curIndex !== this.focusDotIndex) {
                this.control.setDotPos(curPos, curIndex);
            }
        }
    }
    dragEnd() {
        if (!this.states.has(SliderState.Drag)) {
            return false;
        }
        if (this.lazy) {
            this.syncValueByPos();
        }
        setTimeout(() => {
            if (this.included && this.isNotSync) {
                this.control.setValue(this.value);
            }
            else {
                // Sync slider position
                this.control.syncDotsPos();
            }
            this.states.delete(SliderState.Drag);
            // If useKeyboard is true, keep focus status after dragging
            if (!this.useKeyboard) {
                this.states.delete(SliderState.Focus);
            }
            this.$emit('drag-end');
        });
    }
    blurHandle(e) {
        if (!this.states.has(SliderState.Focus) ||
            !this.$refs.container ||
            this.$refs.container.contains(e.target)) {
            return false;
        }
        this.states.delete(SliderState.Focus);
    }
    clickHandle(e) {
        if (!this.clickable) {
            return false;
        }
        if (this.states.has(SliderState.Drag)) {
            return;
        }
        this.setScale();
        const pos = this.getPosByEvent(e);
        this.setValueByPos(pos);
    }
    focus(index = 0) {
        this.states.add(SliderState.Focus);
        this.focusDotIndex = index;
    }
    blur() {
        this.states.delete(SliderState.Focus);
    }
    getValue() {
        const values = this.control.dotsValue;
        return values.length === 1 ? values[0] : values;
    }
    getIndex() {
        const indexes = this.control.dotsIndex;
        return indexes.length === 1 ? indexes[0] : indexes;
    }
    setValueByPos(pos) {
        const index = this.control.getRecentDot(pos);
        if (this.isDisabledByDotIndex(index)) {
            return false;
        }
        this.focusDotIndex = index;
        this.control.setDotPos(pos, index);
        this.syncValueByPos();
        if (this.useKeyboard) {
            this.states.add(SliderState.Focus);
        }
        setTimeout(() => {
            if (this.included && this.isNotSync) {
                this.control.setValue(this.value);
            }
            else {
                this.control.syncDotsPos();
            }
        });
    }
    keydownHandle(e) {
        if (!this.useKeyboard || !this.states.has(SliderState.Focus)) {
            return false;
        }
        const handleFunc = getKeyboardHandleFunc(e, {
            direction: this.direction,
            max: this.control.total,
            min: 0,
        });
        if (handleFunc) {
            e.preventDefault();
            const index = this.control.getIndexByValue(this.control.dotsValue[this.focusDotIndex]);
            const newIndex = handleFunc(index);
            const pos = this.control.parseValue(this.control.getValueByIndex(newIndex));
            this.isCrossDot(pos);
            this.control.setDotPos(pos, this.focusDotIndex);
            this.syncValueByPos();
        }
    }
    getPosByEvent(e) {
        return getPos(e, this.$el, this.isReverse)[this.isHorizontal ? 'x' : 'y'] / this.scale;
    }
    renderSlot(name, data, defaultSlot, isDefault) {
        const scopedSlot = this.$scopedSlots[name];
        return scopedSlot ? (isDefault ? (scopedSlot(data)) : (<template slot={name}>{scopedSlot(data)}</template>)) : (defaultSlot);
    }
    render() {
        return (<div ref="container" class={this.containerClasses} style={this.containerStyles} aria-hidden={true} onClick={this.clickHandle}>
        
        <div class="vue-slider-rail" style={this.railStyle}>
          {this.processBaseStyleArray.map((baseStyle, index) => (<div class="vue-slider-process" key={`process-${index}`} style={[baseStyle, this.processStyle]}/>))}
          
          {this.marks ? (<div class="vue-slider-marks">
              {this.control.markList.map(mark => this.renderSlot('mark', mark, <vue-slider-mark mark={mark} hideLabel={this.hideLabel} style={{
            [this.isHorizontal ? 'height' : 'width']: '100%',
            [this.isHorizontal ? 'width' : 'height']: this.tailSize,
            [this.mainDirection]: `${mark.pos}%`,
        }} stepStyle={this.stepStyle} stepActiveStyle={this.stepActiveStyle} labelStyle={this.labelStyle} labelActiveStyle={this.labelActiveStyle} onPressLabel={(pos) => this.setValueByPos(pos)}>
                    {this.renderSlot('step', mark, null)}
                    {this.renderSlot('label', mark, null)}
                  </vue-slider-mark>, true))}
            </div>) : null}
          
          {this.dots.map((dot, index) => (<vue-slider-dot ref={`dot-${index}`} key={`dot-${index}`} value={dot.value} disabled={dot.disabled} focus={dot.focus} dot-style={[
            dot.style,
            dot.disabled ? dot.disabledStyle : null,
            dot.focus ? dot.focusStyle : null,
        ]} tooltip={dot.tooltip || this.tooltip} tooltip-style={[
            this.tooltipStyle,
            dot.tooltipStyle,
            dot.disabled ? dot.tooltipDisabledStyle : null,
            dot.focus ? dot.tooltipFocusStyle : null,
        ]} tooltip-formatter={this.tooltipFormatter} tooltip-placement={this.tooltipDirections[index]} style={[
            this.dotBaseStyle,
            {
                [this.mainDirection]: `${dot.pos}%`,
                transition: `${this.mainDirection} ${this.animateTime}s`,
            },
        ]} onDrag-start={() => this.dragStart(index)}>
              {this.renderSlot('dot', dot, null)}
              {this.renderSlot('tooltip', dot, null)}
            </vue-slider-dot>))}
        </div>
        
        {this.dots.length === 1 && !this.data ? (<input class="vue-slider-sr-only" type="range" value={this.value} min={this.min} max={this.max}/>) : null}
      </div>);
    }
};
tslib_1.__decorate([
    Model('change', { default: 0 })
], VueSlider.prototype, "value", void 0);
tslib_1.__decorate([
    Prop({
        default: 'ltr',
        validator: dir => ['ltr', 'rtl', 'ttb', 'btt'].indexOf(dir) > -1,
    })
], VueSlider.prototype, "direction", void 0);
tslib_1.__decorate([
    Prop({ type: [Number, String] })
], VueSlider.prototype, "width", void 0);
tslib_1.__decorate([
    Prop({ type: [Number, String] })
], VueSlider.prototype, "height", void 0);
tslib_1.__decorate([
    Prop({ default: 14 })
], VueSlider.prototype, "dotSize", void 0);
tslib_1.__decorate([
    Prop({ type: Number, default: 0 })
], VueSlider.prototype, "min", void 0);
tslib_1.__decorate([
    Prop({ type: Number, default: 100 })
], VueSlider.prototype, "max", void 0);
tslib_1.__decorate([
    Prop({ type: Number, default: 1 })
], VueSlider.prototype, "interval", void 0);
tslib_1.__decorate([
    Prop({ type: Boolean, default: false })
], VueSlider.prototype, "disabled", void 0);
tslib_1.__decorate([
    Prop({ type: Boolean, default: true })
], VueSlider.prototype, "clickable", void 0);
tslib_1.__decorate([
    Prop({ type: Number, default: 0.5 })
], VueSlider.prototype, "duration", void 0);
tslib_1.__decorate([
    Prop(Array)
], VueSlider.prototype, "data", void 0);
tslib_1.__decorate([
    Prop({ type: Boolean, default: false })
], VueSlider.prototype, "lazy", void 0);
tslib_1.__decorate([
    Prop({
        type: String,
        validator: val => ['none', 'always', 'focus'].includes(val),
        default: 'focus',
    })
], VueSlider.prototype, "tooltip", void 0);
tslib_1.__decorate([
    Prop({
        type: String,
        validator: val => ['top', 'right', 'bottom', 'left'].includes(val),
    })
], VueSlider.prototype, "tooltipPlacement", void 0);
tslib_1.__decorate([
    Prop({ type: [String, Function] })
], VueSlider.prototype, "tooltipFormatter", void 0);
tslib_1.__decorate([
    Prop({ type: Boolean, default: false })
], VueSlider.prototype, "useKeyboard", void 0);
tslib_1.__decorate([
    Prop({ type: Boolean, default: true })
], VueSlider.prototype, "enableCross", void 0);
tslib_1.__decorate([
    Prop({ type: Boolean, default: false })
], VueSlider.prototype, "fixed", void 0);
tslib_1.__decorate([
    Prop({ type: Boolean, default: true })
], VueSlider.prototype, "order", void 0);
tslib_1.__decorate([
    Prop(Number)
], VueSlider.prototype, "minRange", void 0);
tslib_1.__decorate([
    Prop(Number)
], VueSlider.prototype, "maxRange", void 0);
tslib_1.__decorate([
    Prop({ type: [Boolean, Object, Array, Function], default: false })
], VueSlider.prototype, "marks", void 0);
tslib_1.__decorate([
    Prop({ type: [Boolean, Function], default: true })
], VueSlider.prototype, "process", void 0);
tslib_1.__decorate([
    Prop(Boolean)
], VueSlider.prototype, "included", void 0);
tslib_1.__decorate([
    Prop(Boolean)
], VueSlider.prototype, "hideLabel", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "dotOptions", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "railStyle", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "processStyle", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "dotStyle", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "tooltipStyle", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "stepStyle", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "stepActiveStyle", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "labelStyle", void 0);
tslib_1.__decorate([
    Prop()
], VueSlider.prototype, "labelActiveStyle", void 0);
tslib_1.__decorate([
    Watch('value')
], VueSlider.prototype, "onValueChanged", null);
VueSlider = tslib_1.__decorate([
    Component({
        data() {
            return {
                control: null,
            };
        },
        components: {
            VueSliderDot,
            VueSliderMark,
        },
        inheritAttrs: false,
    })
], VueSlider);
export default VueSlider;
//# sourceMappingURL=vue-slider.jsx.map